@@ -9,8 +9,7 @@ enum sfd_type {
 	LOG_FD_OFF,
 	IMG_FD_OFF,
 	PROC_FD_OFF,	/* fd with /proc for all proc_ calls */
-	CTL_TTY_OFF,
-	SELF_STDIN_OFF,
+	PROC_PID_FD_OFF,
 	CR_PROC_FD_OFF, /* some other's proc fd.
 			 *  For dump -- target ns' proc
 			 *  For restore -- CRIU ns' proc
@@ -27,14 +26,17 @@ enum sfd_type {
 	SERVICE_FD_MAX
 };
 
-extern int clone_service_fd(int id);
+struct pstree_item;
+extern bool sfds_protected;
+
+extern void set_proc_self_fd(int fd);
+extern int clone_service_fd(struct pstree_item *me);
 extern int init_service_fd(void);
 extern int get_service_fd(enum sfd_type type);
-extern int reserve_service_fd(enum sfd_type type);
 extern int install_service_fd(enum sfd_type type, int fd);
 extern int close_service_fd(enum sfd_type type);
 extern bool is_service_fd(int fd, enum sfd_type type);
 extern bool is_any_service_fd(int fd);
-extern int service_fd_min_fd(void);
+extern int service_fd_min_fd(struct pstree_item *);
 
 #endif /* __CR_SERVICE_FD_H__ */
