@@ -26,6 +26,8 @@ enum {
 	CNT_PAGES_SKIPPED_PARENT,
 	CNT_PAGES_WRITTEN,
 	CNT_PAGES_LAZY,
+	CNT_PAGE_PIPES,
+	CNT_PAGE_PIPE_BUFS,
 
 	DUMP_CNT_NR_STATS,
 };
